@@ -1,8 +1,12 @@
+#' @importFrom forecast Acf
+#' @importFrom stats glm lm constrOptim optim var na.omit
+#' @importFrom JuliaConnectoR juliaCall
+#' @importFrom numDeriv grad hessian
 cocoReg_cov <- function(type, order, data, xreg, seasonality = c(1, 2), mu = 1e-4,
                         outer.it = 500, outer.eps = 1e-10, optim_control = FALSE, constrained.optim = TRUE, b.beta = -10,
                         start = NULL, start.val.adjust = TRUE, method_optim= "Nelder-Mead",
                         replace.start.val = 1e-5, iteration.start.val = 0.99,
-                        method.hessian = "Richardson", julia_installed=FALSE, ...) {
+                        method.hessian = "Richardson", julia_installed=FALSE, link_function="log", ...) {
   start_time <- Sys.time()
 
   if (is.data.frame(data)) {
@@ -72,8 +76,17 @@ cocoReg_cov <- function(type, order, data, xreg, seasonality = c(1, 2), mu = 1e-
   unconstrained.optim.upper <- Inf
   
   df_covariates <- as.data.frame(cbind(data,xreg))
-  covariates_glm_fit <- stats::glm(data ~ -1 + ., family="poisson", data=df_covariates)
-  starting_values_covariates <- stats::na.omit(covariates_glm_fit$coefficients)
+  
+  if (link_function=="log"){
+    covariates_glm_fit <- stats::glm(data ~ -1 + ., family="poisson", data=df_covariates)
+    starting_values_covariates <- stats::na.omit(covariates_glm_fit$coefficients)
+  } else if (link_function %in% c("identity", "relu")) {
+    covariates_glm_fit <- stats::lm(data ~ -1 + ., data=df_covariates)
+    starting_values_covariates <- stats::na.omit(covariates_glm_fit$coefficients)
+    starting_values_covariates[starting_values_covariates <= 0] <- 1e-5
+  } else {
+    stop("Choose either identity, log, or relu as link function.")
+  }
 
   # PAR1
   if ((type == "Poisson") & (order == 1)) {
@@ -90,7 +103,7 @@ cocoReg_cov <- function(type, order, data, xreg, seasonality = c(1, 2), mu = 1e-
 
         T <- length(data)
         xreg_matrix <- data.matrix(xreg)
-        mlef <- likelihoodGP1cov(20, alpha, eta, vec_lambda, T, seasonality[1], data, xreg_matrix)
+        mlef <- likelihoodGP1cov(20, alpha, eta, vec_lambda, T, seasonality[1], data, xreg_matrix, link_function)
         return(mlef)
       } # end function
 
@@ -176,7 +189,7 @@ cocoReg_cov <- function(type, order, data, xreg, seasonality = c(1, 2), mu = 1e-
       names(pars) <- c("alpha", lambs)
       T <- length(data)
       xreg_matrix <- data.matrix(xreg)
-      likelihood <- -likelihoodGP1cov(20, pars[1], 0, pars[-c(1, 2)], T, seasonality[1], data, xreg_matrix)
+      likelihood <- -likelihoodGP1cov(20, pars[1], 0, pars[-c(1, 2)], T, seasonality[1], data, xreg_matrix, link_function)
 
       h <- function(par) {
         alpha <- par[1]
@@ -192,7 +205,7 @@ cocoReg_cov <- function(type, order, data, xreg, seasonality = c(1, 2), mu = 1e-
 
         T <- length(data)
         xreg_matrix <- data.matrix(xreg)
-        mlef <- likelihoodGP1cov(20, alpha, eta, vec_lambda, T, seasonality[1], data, xreg_matrix)
+        mlef <- likelihoodGP1cov(20, alpha, eta, vec_lambda, T, seasonality[1], data, xreg_matrix, link_function)
         return(mlef)
       } # end ml function
 
@@ -210,7 +223,7 @@ cocoReg_cov <- function(type, order, data, xreg, seasonality = c(1, 2), mu = 1e-
       
       if (julia_installed) {
         addJuliaFunctions()
-        julia_reg <- JuliaConnectoR::juliaCall("create_julia_dict", 
+        julia_reg <- JuliaConnectoR::juliaCall("Coconots.create_julia_dict", 
                                                list("parameter", "covariance_matrix", "log_likelihood",
                                                     "type", "order", "data", "covariates",
                                                     "link", "starting_values", "optimizer", 
@@ -218,7 +231,7 @@ cocoReg_cov <- function(type, order, data, xreg, seasonality = c(1, 2), mu = 1e-
                                                     "max_loop"),
                                                list(pars, inv_hes, likelihood,
                                                     type, order, data, xreg,
-                                                    NULL, NULL, NULL, NULL, NULL,
+                                                    link_function, NULL, NULL, NULL, NULL,
                                                     NULL, NULL))
       } else {julia_reg = NULL}
       
@@ -228,7 +241,7 @@ cocoReg_cov <- function(type, order, data, xreg, seasonality = c(1, 2), mu = 1e-
         "hessian" = hes, "inv hessian" = inv_hes, "se" = se,
         "ts" = data, "cov" = xreg, "type" = "Poisson", "order" = 1,
         "seasonality" = seasonality, "likelihood" = likelihood,
-        "duration" = end_time - start_time, julia_reg = julia_reg
+        "duration" = end_time - start_time, julia_reg = julia_reg, "link_function"=link_function
       )
       
       return(list_func)
@@ -253,7 +266,7 @@ cocoReg_cov <- function(type, order, data, xreg, seasonality = c(1, 2), mu = 1e-
 
         T <- length(data)
         xreg_matrix <- data.matrix(xreg)
-        mlef <- likelihoodGP1cov(20, alpha, eta, vec_lambda, T, seasonality[1], data, xreg_matrix)
+        mlef <- likelihoodGP1cov(20, alpha, eta, vec_lambda, T, seasonality[1], data, xreg_matrix, link_function)
         return(mlef)
       } # end function
 
@@ -348,7 +361,7 @@ cocoReg_cov <- function(type, order, data, xreg, seasonality = c(1, 2), mu = 1e-
       names(pars) <- c("alpha", "eta", lambs)
       T <- length(data)
       xreg_matrix <- data.matrix(xreg)
-      likelihood <- -likelihoodGP1cov(20, pars[1], pars[2], pars[-c(1, 2)], T, seasonality[1], data, xreg_matrix)
+      likelihood <- -likelihoodGP1cov(20, pars[1], pars[2], pars[-c(1, 2)], T, seasonality[1], data, xreg_matrix, link_function)
 
       h <- function(par) {
         alpha <- par[1]
@@ -363,7 +376,7 @@ cocoReg_cov <- function(type, order, data, xreg, seasonality = c(1, 2), mu = 1e-
 
         T <- length(data)
         xreg_matrix <- data.matrix(xreg)
-        mlef <- likelihoodGP1cov(20, alpha, eta, vec_lambda, T, seasonality[1], data, xreg_matrix)
+        mlef <- likelihoodGP1cov(20, alpha, eta, vec_lambda, T, seasonality[1], data, xreg_matrix, link_function)
         return(mlef)
       } # end ml function
 
@@ -387,7 +400,7 @@ cocoReg_cov <- function(type, order, data, xreg, seasonality = c(1, 2), mu = 1e-
       
       if (julia_installed) {
         addJuliaFunctions()
-        julia_reg <- JuliaConnectoR::juliaCall("create_julia_dict", 
+        julia_reg <- JuliaConnectoR::juliaCall("Coconots.create_julia_dict", 
                                                list("parameter", "covariance_matrix", "log_likelihood",
                                                     "type", "order", "data", "covariates",
                                                     "link", "starting_values", "optimizer", 
@@ -395,7 +408,7 @@ cocoReg_cov <- function(type, order, data, xreg, seasonality = c(1, 2), mu = 1e-
                                                     "max_loop"),
                                                list(pars, inv_hes, likelihood,
                                                     type, order, data, xreg,
-                                                    NULL, NULL, NULL, NULL, NULL,
+                                                    link_function, NULL, NULL, NULL, NULL,
                                                     NULL, NULL))
       } else {julia_reg = NULL}
       
@@ -404,7 +417,7 @@ cocoReg_cov <- function(type, order, data, xreg, seasonality = c(1, 2), mu = 1e-
         "hessian" = hes, "inv hessian" = inv_hes, "se" = se,
         "ts" = data, "cov" = xreg, "type" = "GP", "order" = 1,
         "seasonality" = seasonality, "likelihood" = likelihood, "duration" = end_time - start_time,
-        julia_reg = julia_reg
+        julia_reg = julia_reg, "link_function"=link_function
       )
  
       return(list_func)
@@ -432,7 +445,7 @@ cocoReg_cov <- function(type, order, data, xreg, seasonality = c(1, 2), mu = 1e-
 
         T <- length(data)
         xreg_matrix <- data.matrix(xreg)
-        mlef <- likelihoodGP2cov(20, alpha1, alpha2, alpha3, eta, vec_lambda, T, seasonality[1], seasonality[2], data, xreg_matrix)
+        mlef <- likelihoodGP2cov(20, alpha1, alpha2, alpha3, eta, vec_lambda, T, seasonality[1], seasonality[2], data, xreg_matrix, link_function)
 
         return(mlef)
       } # end function
@@ -561,7 +574,7 @@ cocoReg_cov <- function(type, order, data, xreg, seasonality = c(1, 2), mu = 1e-
       names(pars) <- c("alpha1", "alpha2", "alpha3", lambs)
       T <- length(data)
       xreg_matrix <- data.matrix(xreg)
-      likelihood <- -likelihoodGP2cov(20, pars[1], pars[2], pars[3], 0, pars[-c(1:4)], T, seasonality[1], seasonality[2], data, xreg_matrix)
+      likelihood <- -likelihoodGP2cov(20, pars[1], pars[2], pars[3], 0, pars[-c(1:4)], T, seasonality[1], seasonality[2], data, xreg_matrix, link_function)
 
 
       h <- function(par) {
@@ -581,7 +594,7 @@ cocoReg_cov <- function(type, order, data, xreg, seasonality = c(1, 2), mu = 1e-
 
         T <- length(data)
         xreg_matrix <- data.matrix(xreg)
-        mlef <- likelihoodGP2cov(20, alpha1, alpha2, alpha3, eta, vec_lambda, T, seasonality[1], seasonality[2], data, xreg_matrix)
+        mlef <- likelihoodGP2cov(20, alpha1, alpha2, alpha3, eta, vec_lambda, T, seasonality[1], seasonality[2], data, xreg_matrix, link_function)
         return(mlef)
       } # end ml function
 
@@ -605,7 +618,7 @@ cocoReg_cov <- function(type, order, data, xreg, seasonality = c(1, 2), mu = 1e-
       
       if (julia_installed) {
         addJuliaFunctions()
-        julia_reg <- JuliaConnectoR::juliaCall("create_julia_dict", 
+        julia_reg <- JuliaConnectoR::juliaCall("Coconots.create_julia_dict", 
                                                list("parameter", "covariance_matrix", "log_likelihood",
                                                     "type", "order", "data", "covariates",
                                                     "link", "starting_values", "optimizer", 
@@ -613,7 +626,7 @@ cocoReg_cov <- function(type, order, data, xreg, seasonality = c(1, 2), mu = 1e-
                                                     "max_loop"),
                                                list(pars, inv_hes, likelihood,
                                                     type, order, data, xreg,
-                                                    NULL, NULL, NULL, NULL, NULL,
+                                                    link_function, NULL, NULL, NULL, NULL,
                                                     NULL, NULL))
       } else {julia_reg = NULL}
       
@@ -622,7 +635,7 @@ cocoReg_cov <- function(type, order, data, xreg, seasonality = c(1, 2), mu = 1e-
         "gradient" = gra, "hessian" = hes, "inv hessian" = inv_hes,
         "se" = se, "ts" = data, "cov" = xreg, "type" = "Poisson",
         "order" = 2, "seasonality" = seasonality, "likelihood" = likelihood,
-        "duration" = end_time - start_time, julia_reg = julia_reg
+        "duration" = end_time - start_time, julia_reg = julia_reg, "link_function"=link_function
       )
 
       return(list_func)
@@ -654,7 +667,7 @@ cocoReg_cov <- function(type, order, data, xreg, seasonality = c(1, 2), mu = 1e-
 
         T <- length(data)
         xreg_matrix <- data.matrix(xreg)
-        mlef <- likelihoodGP2cov(20, alpha1, alpha2, alpha3, eta, vec_lambda, T, seasonality[1], seasonality[2], data, xreg_matrix)
+        mlef <- likelihoodGP2cov(20, alpha1, alpha2, alpha3, eta, vec_lambda, T, seasonality[1], seasonality[2], data, xreg_matrix, link_function)
 
         return(mlef)
       } # end function
@@ -811,7 +824,7 @@ cocoReg_cov <- function(type, order, data, xreg, seasonality = c(1, 2), mu = 1e-
       names(pars) <- c("alpha1", "alpha2", "alpha3", "eta", lambs)
       T <- length(data)
       xreg_matrix <- data.matrix(xreg)
-      likelihood <- -likelihoodGP2cov(20, pars[1], pars[2], pars[3], pars[4], pars[-c(1:4)], T, seasonality[1], seasonality[2], data, xreg_matrix)
+      likelihood <- -likelihoodGP2cov(20, pars[1], pars[2], pars[3], pars[4], pars[-c(1:4)], T, seasonality[1], seasonality[2], data, xreg_matrix, link_function)
 
 
       h <- function(par) {
@@ -832,7 +845,7 @@ cocoReg_cov <- function(type, order, data, xreg, seasonality = c(1, 2), mu = 1e-
 
         T <- length(data)
         xreg_matrix <- data.matrix(xreg)
-        mlef <- likelihoodGP2cov(20, alpha1, alpha2, alpha3, eta, vec_lambda, T, seasonality[1], seasonality[2], data, xreg_matrix)
+        mlef <- likelihoodGP2cov(20, alpha1, alpha2, alpha3, eta, vec_lambda, T, seasonality[1], seasonality[2], data, xreg_matrix, link_function)
 
 
         return(mlef)
@@ -862,7 +875,7 @@ cocoReg_cov <- function(type, order, data, xreg, seasonality = c(1, 2), mu = 1e-
       
       if (julia_installed) {
         addJuliaFunctions()
-        julia_reg <- JuliaConnectoR::juliaCall("create_julia_dict", 
+        julia_reg <- JuliaConnectoR::juliaCall("Coconots.create_julia_dict", 
                                                list("parameter", "covariance_matrix", "log_likelihood",
                                                     "type", "order", "data", "covariates",
                                                     "link", "starting_values", "optimizer", 
@@ -870,7 +883,7 @@ cocoReg_cov <- function(type, order, data, xreg, seasonality = c(1, 2), mu = 1e-
                                                     "max_loop"),
                                                list(pars, inv_hes, likelihood,
                                                     type, order, data, xreg,
-                                                    NULL, NULL, NULL, NULL, NULL,
+                                                    link_function, NULL, NULL, NULL, NULL,
                                                     NULL, NULL))
       } else {julia_reg = NULL}
       
@@ -879,7 +892,7 @@ cocoReg_cov <- function(type, order, data, xreg, seasonality = c(1, 2), mu = 1e-
         "gradient" = gra, "hessian" = hes, "inv hessian" = inv_hes,
         "se" = se, "ts" = data, "cov" = xreg, "type" = "GP", "order" = 2,
         "seasonality" = seasonality, "likelihood" = likelihood,
-        "duration" = end_time - start_time, julia_reg = julia_reg
+        "duration" = end_time - start_time, julia_reg = julia_reg, "link_function"=link_function
       )
 
       return(list_func)
