@@ -7,12 +7,13 @@
 #' @param par numeric vector, the parameters of the model, the number of elements
 #'  in the vector depends on the type and order specified. 
 #' @param length integer, the number of observations in the generated time series
-#' @param xreg data.frame, data frame of control variables
-#' @param init numeric vector, initial data to use, default is NULL. See details
+#' @param xreg data frame of control variables (defaul: NULL)
+#' @param init numeric vector, initial data to use (default: NULL). See details
 #' for more information on the usage.
-#' @param julia If TRUE, the Julia implementation is used. In this case, init is ignored but it might be faster.
-#' @param julia_seed Seed for the Julia implementation. Only used if Julia equals TRUE.
-#' @return a vector of the simulated time series. 
+#' @param julia If TRUE, the \proglang{julia} implementation is used. In this case, init is ignored but it might be faster (default: FALSE).
+#' @param link_function Specifies the link function for the conditional mean of the innovation (\eqn{\lambda}). The default is `log`, but other available options include `identity` and `relu`. This parameter is applicable only when covariates are used. Note that using the `identity` link function may result in \eqn{\lambda} becoming negative. To prevent this, ensure all covariates are positive and restrict the parameter \eqn{\beta} to positive values.
+#' @param julia_seed Seed for the \proglang{julia} implementation. Only used if \proglang{julia} equals TRUE.
+#' @return a vector of the simulated time series 
 #' @details The function checks for valid input of the type, order, parameters, and initial data
 #' before generating the time series.
 #'
@@ -36,12 +37,10 @@
 #' 
 #' # Simulate using the RCPP implementation
 #' data_rcpp <- cocoSim(order = 1, type = "Poisson", par = c(lambda, alpha), length = 100)
-#' # Simulate using the Julia implementation
-#' data_julia <- cocoSim(order = 1, type = "Poisson", par = c(lambda, alpha), length = 100)
 #' @export
 
 cocoSim <- function(type, order, par, length, xreg = NULL, init = NULL, 
-                    julia=FALSE, julia_seed = NULL) {
+                    julia=FALSE, julia_seed = NULL, link_function="log") {
   seasonality <- c(1, 2) #will be used as argument in future versions
   
   
@@ -84,7 +83,7 @@ cocoSim <- function(type, order, par, length, xreg = NULL, init = NULL,
       if (!is.null(julia_seed)){
         setJuliaSeed(julia_seed)
       }
-      return(cocoSimJulia(type, order, par, length, xreg))
+      return(cocoSimJulia(type, order, par, length, xreg, link_function))
     }
     
     size <- length + length_burn_in + init_add
@@ -117,12 +116,12 @@ cocoSim <- function(type, order, par, length, xreg = NULL, init = NULL,
     
     if (julia){
       setJuliaSeed(julia_seed)
-      return(cocoSimJulia(type, order, par, length, xreg))
+      return(cocoSimJulia(type, order, par, length, xreg, link_function))
     }
     size <- length + init_add
     output <- cocoSim_cov(
       type = type, order = order, par = par, size = size, xreg = xreg,
-      seasonality = seasonality, init = init
+      seasonality = seasonality, init = init, link_function=link_function
     )
     
   }
